from ldnode.calculator.schemas import (
    CBBacktestParameters,
    STKBacktestParameters,
    ETFBacktestParameters,
)
from .common import TaskBaseParameters


class BaseGlanceParameters(TaskBaseParameters):
    """ 策略收益计算参数 """
    created_date: str # 创建日期


class CBGlanceParameters(BaseGlanceParameters):
    """ 转债策略收益计算参数 """
    strategy_config: CBBacktestParameters
    

class STKGlanceParameters(BaseGlanceParameters):
    """ 股票策略收益计算参数 """
    strategy_config: STKBacktestParameters


class ETFGlanceParameters(BaseGlanceParameters):
    """ 基金策略收益计算参数 """
    strategy_config: ETFBacktestParameters
