from typing import TypedDict

from ldnode.calculator.schemas import GridBacktestParameters, GridBacktestResult
from .common import TaskBaseParameters


class CBGridBacktestParameters(TaskBaseParameters):
    """ 转债网格回测参数 """
    strategy_config: GridBacktestParameters # 策略配置


class CBGridBacktestResponse(TypedDict):
    """ 转债网格回测响应 """
    evaluation: GridBacktestResult
    history_position: bytes # 历史持仓
    