import polars as pl


def planning_merge_benchmark(
    res_lf: pl.LazyFrame,
    benchmark_lf: pl.LazyFrame,
) -> pl.LazyFrame:
    """ 规划合并基准benchmark """
    return res_lf.join(benchmark_lf, on='trade_date', how='left')
