from enum import Enum, StrEnum


class HyperparameterSearchMode(Enum):
    """ 超参数搜索模式 """
    GRID = "grid"
    BAYESIAN = "bayesian"


class HyperTuningScopeArgName(StrEnum):
    """ 超参数调优范围参数名称 """
    RANK_WEIGHT = "rank_weight"
    RANK_FACTOR = "rank_factor"
    FILTER_VALUE = "filter_value"
    STOP_PROFIT_LOSS = "stop_profit_loss"
